@@ -58,6 +58,42 @@ import pandas
 from freesurfer_stats.version import __version__
 
 
+def _get_filepath_or_buffer(
+    path: typing.Union[str, pathlib.Path]
+) -> typing.Tuple[
+    typing.Any, bool  # pandas._typing.FileOrBuffer, bool)
+]:  # pragma: no cover
+    # can't check coverage due to pandas version branching.
+    # pipeline tests against multiple pandas versions.
+    if not hasattr(pandas.io.common, "get_filepath_or_buffer"):
+        # pandas.io.common.get_filepath_or_buffer was made private in v1.2.0:
+        # https://github.com/pandas-dev/pandas/commit/6d1541e1782a7b94797d5432922e64a97934cfa4#diff-934d8564d648e7521db673c6399dcac98e45adfd5230ba47d3aabfcc21979febL247
+        # semver?!? breaking change not even mentioned in changelog:
+        # https://pandas.pydata.org/pandas-docs/stable/whatsnew/v1.2.0.html
+        # new wrapper: get_handle
+        # https://github.com/pandas-dev/pandas/blob/v1.2.0/pandas/io/common.py#L490
+        # pandas v1.1's get_handle does not yet support urls
+        # pylint: disable=no-member; for python<v1.2.0
+        io_handle = pandas.io.common.get_handle(path, "r")
+        return io_handle.handle, True
+    # path_or_buffer: typing.Union[str, pathlib.Path, typing.IO[typing.AnyStr],
+    #                              s3fs.S3File, gcsfs.GCSFile]
+    # https://github.com/pandas-dev/pandas/blob/v0.25.3/pandas/io/parsers.py#L436
+    # https://github.com/pandas-dev/pandas/blob/v0.25.3/pandas/_typing.py#L30
+    # pylint: disable=no-member; for python>=v1.2.0
+    (path_or_buffer, _, _, *instructions) = pandas.io.common.get_filepath_or_buffer(
+        path
+    )
+    if instructions:
+        # https://github.com/pandas-dev/pandas/blob/v0.25.3/pandas/io/common.py#L171
+        assert len(instructions) == 1, instructions
+        should_close = instructions[0]
+    else:
+        # https://github.com/pandas-dev/pandas/blob/v0.21.0/pandas/io/common.py#L171
+        should_close = hasattr(path_or_buffer, "close")
+    return path_or_buffer, should_close
+
+
 class CorticalParcellationStats:
 
     _HEMISPHERE_PREFIX_TO_SIDE = {"lh": "left", "rh": "right"}
@@ -77,7 +113,7 @@ class CorticalParcellationStats:
 
     @property
     def hemisphere(self) -> str:
-        return self._HEMISPHERE_PREFIX_TO_SIDE[self.headers["hemi"]]
+        return self._HEMISPHERE_PREFIX_TO_SIDE[typing.cast(str, self.headers["hemi"])]
 
     @staticmethod
     def _read_header_line(stream: typing.TextIO) -> str:
@@ -87,7 +123,7 @@ class CorticalParcellationStats:
 
     @classmethod
     def _read_column_header_line(
-        cls, stream: typing.TextIO,
+        cls, stream: typing.TextIO
     ) -> typing.Tuple[int, str, str]:
         line = cls._read_header_line(stream)
         assert line.startswith("TableCol"), line
@@ -102,26 +138,31 @@ class CorticalParcellationStats:
             if line.startswith("Measure"):
                 break
             if line:
-                attr_name, attr_value = line.split(" ", maxsplit=1)
-                attr_value = attr_value.lstrip()
+                attr_name, attr_value_str = line.split(" ", maxsplit=1)
+                attr_value_str = attr_value_str.lstrip()
                 if attr_name in ["cvs_version", "mrisurf.c-cvs_version"]:
-                    attr_value = attr_value.strip("$").rstrip()
-                if attr_name == "CreationTime":
+                    attr_value = typing.cast(
+                        typing.Union[str, datetime.datetime],
+                        attr_value_str.strip("$").rstrip(),
+                    )
+                elif attr_name == "CreationTime":
                     attr_dt = datetime.datetime.strptime(
-                        attr_value, "%Y/%m/%d-%H:%M:%S-%Z",
+                        attr_value_str, "%Y/%m/%d-%H:%M:%S-%Z"
                     )
                     if attr_dt.tzinfo is None:
-                        assert attr_value.endswith("-GMT")
+                        assert attr_value_str.endswith("-GMT")
                         attr_dt = attr_dt.replace(tzinfo=datetime.timezone.utc)
                     attr_value = attr_dt
-                if attr_name == "AnnotationFileTimeStamp":
+                elif attr_name == "AnnotationFileTimeStamp":
                     attr_value = datetime.datetime.strptime(
-                        attr_value, "%Y/%m/%d %H:%M:%S",
+                        attr_value_str, "%Y/%m/%d %H:%M:%S"
                     )
+                else:
+                    attr_value = attr_value_str
                 self.headers[attr_name] = attr_value
 
     @classmethod
-    def _format_column_name(cls, name: str, unit: typing.Optional[str]) -> str:
+    def _format_column_name(cls, name: str, unit: str) -> str:
         column_name = name.lower()
         if unit not in ["unitless", "NA"]:
             column_name += "_" + unit
@@ -129,7 +170,7 @@ class CorticalParcellationStats:
 
     @classmethod
     def _parse_whole_brain_measurements_line(
-        cls, line: str,
+        cls, line: str
     ) -> typing.Tuple[str, numpy.ndarray]:
         match = cls._GENERAL_MEASUREMENTS_REGEX.match(line)
         if not match:
@@ -145,11 +186,11 @@ class CorticalParcellationStats:
 
     @classmethod
     def _read_column_attributes(
-        cls, num: int, stream: typing.TextIO,
+        cls, num: int, stream: typing.TextIO
     ) -> typing.List[typing.Dict[str, str]]:
         columns = []
         for column_index in range(1, int(num) + 1):
-            column_attrs = {}
+            column_attrs = {}  # type: typing.Dict[str, str]
             for _ in range(3):
                 column_index_line, key, value = cls._read_column_header_line(stream)
                 assert column_index_line == column_index
@@ -193,31 +234,19 @@ class CorticalParcellationStats:
 
     @classmethod
     def read(cls, path: typing.Union[str, pathlib.Path]) -> "CorticalParcellationStats":
-        # path_or_buffer: typing.Union[str, pathlib.Path, typing.IO[typing.AnyStr],
-        #                              s3fs.S3File, gcsfs.GCSFile]
-        # https://github.com/pandas-dev/pandas/blob/v0.25.3/pandas/io/parsers.py#L436
-        # https://github.com/pandas-dev/pandas/blob/v0.25.3/pandas/_typing.py#L30
-        (
-            path_or_buffer,
-            _,
-            _,
-            *instructions,
-        ) = pandas.io.common.get_filepath_or_buffer(path)
-        # https://github.com/pandas-dev/pandas/blob/v0.25.3/pandas/io/common.py#L171
-        # https://github.com/pandas-dev/pandas/blob/v0.21.0/pandas/io/common.py#L171
-        if instructions:  # pragma: no cover
-            assert len(instructions) == 1, instructions
-            should_close = instructions[0]
-        else:  # pragma: no cover
-            should_close = hasattr(path_or_buffer, "close")
+        path_or_buffer, should_close = _get_filepath_or_buffer(path)
         stats = cls()
-        if hasattr(path_or_buffer, "readline"):
-            # pylint: disable=protected-access
-            stats._read(io.TextIOWrapper(path_or_buffer))
-        else:
-            with open(path_or_buffer, "r") as stream:
-                # pylint: disable=protected-access
-                stats._read(stream)
-        if should_close:
-            path_or_buffer.close()
+        try:  # pragma: no cover
+            # can't check coverage due to pandas version branching.
+            # pylint: disable=protected-access; false-positive for ._read
+            if isinstance(path_or_buffer, io.TextIOWrapper):  # pandas>=v1.2.0
+                stats._read(path_or_buffer)
+            elif hasattr(path_or_buffer, "readline"):
+                stats._read(io.TextIOWrapper(path_or_buffer))
+            else:
+                with open(path_or_buffer, "r") as stream:
+                    stats._read(stream)
+        finally:
+            if should_close:
+                path_or_buffer.close()
         return stats
